	       	.CHIP 8051
;
		.GLOBALS ON		
;
;
;----------------------------------------------------------------
;	  						  	|
;                     SEND MIDI SEQUENCE    		  	| 
;							  	|
;      written by Jiri Cerny in 2004, All rights reserved   	|
;							  	|
;----------------------------------------------------------------
;
;
;               Main Loop Waits for Button Press and then
;
;               sends appropriate MIDI sequence according to
;
;               BCD switch at P1.0 (LSB) to P1.3 (MSB)
;
;               Relay and LED toggle is performed at the same 
;
;               time
;               
;
;-------------- EQUATES & VARIABLES
;
;
SPINIT		EQU 50H		;DEFINE STACK POINTER
SCONINI		EQU 40H		;8 BIT UART MODE
TMODINI		EQU 21H		;TIMER 1-BAUD RATE GENERATOR, TIMER 0-16 BIT
TIMCONST	EQU 0FFH	;BAUDRATE = 32250 kHz
PCONINI		EQU 0		;SMOD=0
;
INDEX           EQU 20H         ;ACTUAL INDEX TO CODETABLE
STATE           EQU 21H         ;BIT 0 HOLDS ACTUAL RELAY AND LED STATE
;
;
;-------------- PORT ASSIGNMENTS
;
;
LED             REG P1.7        ;LED INDICATOR OF RELAY STATE (INVERTED)
RELAY           REG P3.5        ;RELAY STATE TOGGLED AT EACH BUTTON PRESS
BUTTON          REG P3.7        ;LOG 0 MEANS BUTTON PRESSED
;
;-------------- MACROS
;
;
CLI		MACRO
		CLR EA
		ENDM
;
STI		MACRO
		SETB EA
		ENDM
;
STRTTIM1	MACRO
		SETB TR1
		ENDM
;
STOPTIM0	MACRO
		CLR TR0
		ENDM
;
LEDON 	        MACRO
		CLR LED
		ENDM
;
LEDOFF	        MACRO
		SETB LED
		ENDM
;
RELAYON		MACRO
		SETB RELAY
		ENDM
;
RELAYOFF	MACRO
		CLR RELAY
		ENDM
;
;
;
;-------------- MAIN PROGRAM ENTRY
;
;
		ORG 0000H
ENTRY:		JMP PGMBEGIN
;
;
;
;-------------- INTERRUPT VECTORS
;
		ORG 0003H
		JMP EXTINT0
;		
		ORG 000BH
		JMP TIMINT0
;
		ORG 0013H
		JMP EXTINT1
;		
		ORG 001BH
		JMP TIMINT1
;
		ORG 0023H
		JMP SERIALINT
;
		ORG 002BH
		JMP TIMINT2	;80C52 TIMER 2 INTERRUPT VECTOR
;
		ORG 0043H
		JMP EXTINT2	;DS80C320 INT2 VECTOR
;
;
;
;============== INTERRUPT HANDLERS
;
;
;--------------	DUMMY INTERRUPTS
;
SERIALINT:
TIMINT0:
TIMINT1:
TIMINT2:	
EXTINT0:
EXTINT1:
EXTINT2:	RETI
;
;
;============== MAIN PROGRAM BODY
;
;
PGMBEGIN:	CLI                     ;DISABLE INTERRUPTS
		MOV SP,#SPINIT		;INITIALIZE STACK POINTER
		CALL SYSINIT		;INITIALIZE ALL INTERNAL HW
		CALL VARINIT		;INITIALIZE SYSTEM VARIABLES
;
;               MAIN PROGRAM LOOP
;
MAINLOOP:	JB BUTTON,MAINLOOP	;WAIT FOR BUTTON PRESS
                CALL DELAY10            ;CHECK IT
                JB BUTTON,MAINLOOP      ;MAYBE NOISE
                CALL TOGGLE             ;TOGGLE RELAY/LED
                CALL SENDMIDI           ;SEND MIDI SEQUENCE
WRELEASE:       JNB BUTTON,WRELEASE     ;WAIT FOR BUTTON RELEASE
                CALL DELAY10            ;CHECK IT
                JNB BUTTON,WRELEASE     ;MAYBE NOISE
		SJMP MAINLOOP           ;BUTTON RELEASED
;
;==============	UTILITY ROUTINES
;
;
;-------------- SYSTEM INIT
;
;		- INITIALIZE SERIAL PORT AS 8 BIT UART
;		- INITIALIZE TIMER 0 AS 16 BIT TIMER (NOT USED)
;		- INITIALIZE TIMER 1 AS TIMER WITH AUTORELOAD (BAUD
;		  RATE GENERATOR)
;		- SET BAUDRATE TO 31250 kHz
;
;
SYSINIT:	MOV SCON,#SCONINI	;SERIAL INIT
		MOV TMOD,#TMODINI	;TIMER INIT
		MOV TH1,#TIMCONST	;DEFINE DIVISOR
		MOV PCON,#PCONINI	;HALF BAUD RATE
		STRTTIM1		;START BAUD RATE GENERATOR
		STOPTIM0		;DO NOT COUNT NOW	
		RET
;
;-------------- INITIALIZE OUTPUTS AND SYSTEM VARIABLES
;
VARINIT:        RELAYOFF
		LEDOFF
                XRL A,A
                MOV STATE,A
                MOV INDEX,A
	        RET
;
;-------------- TOGGLE RELAY AND LED
;
TOGGLE:         MOV C,STATE.0           ;GET FLAG
                CPL C                   ;TOGGLE IT
                MOV STATE.0,C           ;AND SAVE AGAIN
                JNC SETOFF              ;DO WHAT NECESSARY
                RELAYON                 ;ON
                LEDON
                RET
SETOFF:         RELAYOFF                ;OFF
                LEDOFF
                RET
;
;-------------- SEND MIDI SEQUENCE
;
SENDMIDI:       MOV A,P1                ;GET BCD SWITCH CODE
                CPL A                   ;INVERT IT (SWITCH TO ZERO)
                ANL A,#0FH              ;MASK OUT HI NIBBLE
                MOV R3,A                ;SAVE ACC TO REG
                RL A                    ;MULTIPLY BY TWO
                ADD A,R3                ;MULTIPLY BY THREE = INDEX
                MOV INDEX,A             ;SAVE INDEX TO TABLE
                MOV DPTR,#CODETABLE     ;POINTER TO TABLE START
                MOVC A,@A+DPTR          ;GET FIRST MIDI CHARACTER
                MOV SBUF,A              ;SEND IT OUT
WS1:            JNB TI,WS1              ;WAIT UNTIL DONE
                CLR TI                  ;CLEAR TX FLAG
                MOV A,INDEX             ;GET SAVED INDEX
                INC A                   ;INCREMENT IT
                MOV INDEX,A             ;AND SAVE AGAIN
                MOVC A,@A+DPTR          ;GET NEXT CHARACTER FROM TABLE
                MOV SBUF,A              ;SEND IT OUT
WS2:            JNB TI,WS2              ;WAIT UNTIL DONE
                CLR TI                  ;CLEAR TX FLAG
                MOV A,INDEX             ;GET SAVED INDEX AGAIN
                INC A                   ;INCREMENT IT
                MOV INDEX,A             ;AND SAVE AGAIN            
                MOVC A,@A+DPTR          ;GET LAST CHARACTER FROM TABLE
                MOV SBUF,A              ;SEND IT OUT
WS3:            JNB TI,WS3              ;WAIT UNTIL DONE
                CLR TI                  ;CLEAR TX FLAG
                RET
;
;-------------- WAIT ROUTINES (SOFTWARE IMPLEMENTED)
;
;               SUPPOSED Q=12.0 MHz
;
;		USES R7, R6
;		
;
WAIT100US:	MOV R7,#32	;1 + 32 x (1+2) + 1 + 2 = 100 CYCLES
W1:		NOP		;1  (x 12/12)
		DJNZ R7,W1	;2
		NOP		;1
		RET		;2
;
WAIT5MS:	MOV R7,#8	;1 
W2:		NOP		;1
		MOV R6,#0FFH	;1
W3:		NOP		;1
		DJNZ R6,W3	;2
		DJNZ R7,W2      ;2
		RET
;
;-------------- DELAY 10 ms
;
;
DELAY10:        CALL WAIT5MS
                CALL WAIT5MS
                RET
;
;---------- MIDI SEQUENCES   
;
CODETABLE:  
; 
;CODE=0	    
            	DB 90H, 30H, 7FH    	;Note C3 ON
;
;CODE=1	    
            	DB 90H, 34H, 7FH	;Note E3 ON
;
;CODE=2	    
            	DB 90H, 37H, 7FH	;Note G3 ON
;
;CODE=3	    
            	DB 0B0H, 7BH, 00H	;All notes OFF
;
;CODE=4	    
            	DB 0B0H, 40H, 01H	;Ctrl 64, VAL1
;
;CODE=5	    
            	DB 0B0H, 40H, 02H	;Ctrl 64, VAL2
;
;CODE=6	    
            	DB 0B0H, 40H, 03H	;Ctrl 64, VAL3
;
;CODE=7	    
            	DB 0B0H, 40H, 04H	;Ctrl 64, VAL4
;
;CODE=8	    
            	DB 0B0H, 40H, 05H	;Ctrl 64, VAL5
;
;CODE=9	    
            	DB 0B0H, 40H, 06H	;Ctrl 64, VAL6
;
;-------------- UNUSED CODES
;
;CODE=A	    
            	DB 0B0H, 40H, 07H	;Ctrl 64, VAL7
;CODE=B	    
            	DB 0B0H, 40H, 08H	;Ctrl 64, VAL8
;CODE=C	    
            	DB 0B0H, 40H, 09H	;Ctrl 64, VAL9
;CODE=D	    
            	DB 0B0H, 40H, 0AH	;Ctrl 64, VALA
;CODE=E	    
            	DB 0B0H, 40H, 0BH	;Ctrl 64, VALB
;CODE=F	    
            	DB 0B0H, 40H, 0CH	;Ctrl 64, VALC
;
;--------------
;
		END
;
